### Code accompanying the paper "Cryptographically secure multiparty evaluation
### of system reliability"

################################################################################
### This is the code to be run by the manufacturer of component M.           ###
### First, assure that k$pk, t, kappa and XiL.M have been received from the  ###
### system designer.                                                         ###
### Also assure that XiR.c has been received from manufacturer H.            ###
################################################################################

# Load required libraries.
library("HomomorphicEncryption")
source("utility.R")

# Load the files sent by system designer
load("t_and_kappa.RData")
load("XiL_M.RData")
pk <- loadFHE("pk.fhe")

# Load updated Xi matrix sent by manufacturer H
system("unpigz XiR_c3.fhe.gz")
XiR.c <- loadFHE("XiR_c3.fhe")

# Here the data is simulated, but real test data would be used in practise.
set.seed(which(LETTERS=="M"))
M.data <- rexp(5, rate=0.25)

# First, compute the values to be interacted with XiR.c
updateXi.m <- matrix(0, nrow=length(XiL.M), ncol=length(t))
for(j in 1:length(t)) {
  for(i in 1:length(XiL.M)) {
    updateXi.m[i,j] <- component.posterior(max(XiL.M), XiL.M[i], t[j],
                                           M.data)
  }
}

# Then encrypt under the designer's public key.
updateXi.c <- enc(pk, mu(kappa, updateXi.m))

# Then interact with the encrypted survival signature computed so far.
XiR.c <- XiR.c * updateXi.c

# For safety the following which modify the filesystem have been commented out.
# Review the code to make sure you are happy and then uncomment and run.

# Save Xi then compress ... uncomment the following to run
# NB: XiR.c requires about 11.6GB storage uncompressed or 5.5GB compressed.
#saveFHE(XiR.c, "XiR_c4.fhe")
#system("pigz XiR_c4.fhe")

# Send the XiR.c to manufacturer P, for example over SSH tunnel ... modify the
# destintation, then uncomment the following to run
#system("scp XiR_c4.fhe.gz username@manufacturer-p.com:~")

################################################################################
### This manufacturer now sends XiR.c to the next manufacturer in the        ###
### protocol.  Here, that is manufacturer P.                                 ###
###                                                                          ###
### This is the end of manufacturer M's involvement.                         ###
################################################################################
