### Code accompanying the paper "Cryptographically secure multiparty evaluation
### of system reliability"

################################################################################
### This is the code to be run by the manufacturer of component P.           ###
### First, assure that k$pk, t, kappa and XiL.P have been received from the  ###
### system designer.                                                         ###
### Also assure that XiR.c has been received from manufacturer M.            ###
################################################################################

# Load required libraries.
library("HomomorphicEncryption")
source("utility.R")

# Load the files sent by system designer
load("t_and_kappa.RData")
load("XiL_P.RData")
pk <- loadFHE("pk.fhe")

# Load updated Xi matrix sent by manufacturer M
system("unpigz XiR_c4.fhe.gz")
XiR.c <- loadFHE("XiR_c4.fhe")

# Here the data is simulated, but real test data would be used in practise.
set.seed(which(LETTERS=="P"))
P.data <- rgamma(20, scale=0.9, shape=3.2)

# First, compute the values to be interacted with XiR.c
updateXi.m <- matrix(0, nrow=length(XiL.P), ncol=length(t))
for(j in 1:length(t)) {
  for(i in 1:length(XiL.P)) {
    updateXi.m[i,j] <- component.posterior(max(XiL.P), XiL.P[i], t[j],
                                           P.data)
  }
}

# Then encrypt under the designer's public key.
updateXi.c <- enc(pk, mu(kappa, updateXi.m))

# Then interact with the encrypted survival signature computed so far.
XiR.c <- XiR.c * updateXi.c

# Manufacturer P is the last manufacturer in the protocol in this round, so
# completes the final task of computing the column sums to return to the system
# designer.
xi.c <- colSums(XiR.c)

# For safety the following which modify the filesystem have been commented out.
# Review the code to make sure you are happy and then uncomment and run.

# Save xi then compress ... uncomment the following to run
# NB: xi.c requires about 122.5MB storage uncompressed or 58.4MB compressed.
#saveFHE(xi.c, "xi_c.fhe")
#system("pigz xi_c.fhe")

# Send the xi.c back to the system designer, for example over SSH tunnel ... 
# modify the destintation, then uncomment the following to run
#system("scp xi_c.fhe.gz username@system-designer.com:~")

################################################################################
### This manufacturer now sends xi.c to the system designer, since all the  ###
### other manufacturers have already taken part in the protocol.             ###
###                                                                          ###
### This is the end of manufacturer P's involvement.                         ###
################################################################################
